@@ -416,12 +416,18 @@ _gdk_frame_clock_begin_frame (GdkFrameClock *frame_clock)
   priv->frame_counter++;
   priv->current = (priv->current + 1) % FRAME_HISTORY_MAX_LENGTH;
 
+  /* Try to steal the previous frame timing instead of discarding
+   * and allocating a new one.
+   */
+  if G_LIKELY (priv->n_timings == FRAME_HISTORY_MAX_LENGTH &&
+               _gdk_frame_timings_steal (priv->timings[priv->current],
+                                         priv->frame_counter))
+    return;
+
   if (priv->n_timings < FRAME_HISTORY_MAX_LENGTH)
     priv->n_timings++;
   else
-    {
-      gdk_frame_timings_unref(priv->timings[priv->current]);
-    }
+    gdk_frame_timings_unref (priv->timings[priv->current]);
 
   priv->timings[priv->current] = _gdk_frame_timings_new (priv->frame_counter);
 }
@@ -471,7 +477,7 @@ gdk_frame_clock_get_timings (GdkFrameClock *frame_clock,
  *
  * Returns: (nullable): the #GdkFrameTimings for the frame currently
  *  being processed, or even no frame is being processed, for the
- *  previous frame. Before any frames have been procesed, returns
+ *  previous frame. Before any frames have been processed, returns
  *  %NULL.
  * Since: 3.8
  */
@@ -603,3 +609,45 @@ gdk_frame_clock_get_refresh_info (GdkFrameClock *frame_clock,
       frame_counter--;
     }
 }
+
+void
+_gdk_frame_clock_emit_flush_events (GdkFrameClock *frame_clock)
+{
+  g_signal_emit (frame_clock, signals[FLUSH_EVENTS], 0);
+}
+
+void
+_gdk_frame_clock_emit_before_paint (GdkFrameClock *frame_clock)
+{
+  g_signal_emit (frame_clock, signals[BEFORE_PAINT], 0);
+}
+
+void
+_gdk_frame_clock_emit_update (GdkFrameClock *frame_clock)
+{
+  g_signal_emit (frame_clock, signals[UPDATE], 0);
+}
+
+void
+_gdk_frame_clock_emit_layout (GdkFrameClock *frame_clock)
+{
+  g_signal_emit (frame_clock, signals[LAYOUT], 0);
+}
+
+void
+_gdk_frame_clock_emit_paint (GdkFrameClock *frame_clock)
+{
+  g_signal_emit (frame_clock, signals[PAINT], 0);
+}
+
+void
+_gdk_frame_clock_emit_after_paint (GdkFrameClock *frame_clock)
+{
+  g_signal_emit (frame_clock, signals[AFTER_PAINT], 0);
+}
+
+void
+_gdk_frame_clock_emit_resume_events (GdkFrameClock *frame_clock)
+{
+  g_signal_emit (frame_clock, signals[RESUME_EVENTS], 0);
+}
