@@ -59,9 +59,10 @@
  * int
  * main (int argc, char **argv)
  * {
+ *  GtkWidget *mainwin;
  *   // Initialize i18n support with bindtextdomain(), etc.
  *
- *   ...
+ *   // ...
  *
  *   // Initialize the widget set
  *   gtk_init (&argc, &argv);
@@ -71,7 +72,7 @@
  *
  *   // Set up our GUI elements
  *
- *   ...
+ *   // ...
  *
  *   // Show the application window
  *   gtk_widget_show_all (mainwin);
@@ -809,6 +810,9 @@ post_parse_hook (GOptionContext *context,
 
           return FALSE;
         }
+
+      if (gtk_get_debug_flags () & GTK_DEBUG_INTERACTIVE)
+        gtk_window_set_interactive_debugging (TRUE);
     }
 
   return TRUE;
@@ -947,8 +951,9 @@ gtk_get_option_group (gboolean open_default_display)
  * `--help` output. Note that your program will
  * be terminated after writing out the help output.
  *
- * Returns: %TRUE if the windowing system has been successfully
- *     initialized, %FALSE otherwise
+ * Returns: %TRUE if the commandline arguments (if any) were valid and
+ *     if the windowing system has been successfully initialized,
+ *     %FALSE otherwise
  *
  * Since: 2.6
  */
@@ -988,7 +993,22 @@ gtk_init_with_args (gint                 *argc,
     return FALSE;
 
 done:
-  return GDK_PRIVATE_CALL (gdk_display_open_default) () != NULL;
+  if (GDK_PRIVATE_CALL (gdk_display_open_default) () == NULL)
+    {
+      const char *display_name = gdk_get_display_arg_name ();
+      g_set_error (error,
+                   G_OPTION_ERROR,
+                   G_OPTION_ERROR_FAILED,
+                   _("Cannot open display: %s"),
+                   display_name ? display_name : "" );
+
+      return FALSE;
+    }
+
+  if (gtk_get_debug_flags () & GTK_DEBUG_INTERACTIVE)
+    gtk_window_set_interactive_debugging (TRUE);
+
+  return TRUE;
 }
 
 
@@ -1060,15 +1080,17 @@ gtk_parse_args (int    *argc,
  *     understood by GTK+ are stripped before return.
  *
  * This function does the same work as gtk_init() with only a single
- * change: It does not terminate the program if the windowing system
- * can’t be initialized. Instead it returns %FALSE on failure.
+ * change: It does not terminate the program if the commandline
+ * arguments couldn’t be parsed or the windowing system can’t be
+ * initialized. Instead it returns %FALSE on failure.
  *
  * This way the application can fall back to some other means of
  * communication with the user - for example a curses or command line
  * interface.
  *
- * Returns: %TRUE if the windowing system has been successfully
- *     initialized, %FALSE otherwise
+ * Returns: %TRUE if the commandline arguments (if any) were valid and
+ *     the windowing system has been successfully initialized, %FALSE
+ *     otherwise
  */
 gboolean
 gtk_init_check (int    *argc,
@@ -1300,7 +1322,10 @@ gtk_main (void)
       gdk_threads_leave ();
       g_main_loop_run (loop);
       gdk_threads_enter ();
+
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gdk_flush ();
+      G_GNUC_END_IGNORE_DEPRECATIONS;
     }
 
   main_loops = g_slist_remove (main_loops, loop);
